//*************************************************************************
//
// header file for wave.cpp
//
// Created 2-19-98 by Jim Patchell
//
//*************************************************************************

typedef struct {
	char rID[4];	//should contain the word RIFF
	long rLen;		//length of the data in the next chunk
} RIFF;

typedef struct {
	char wID[4];	//should contain the word WAVE
	char fID[4];	//should contain the word "fmt"
	long fLen;		//length of data in the format chunk
	short int wFormatTag;
	short int nChannels;	//number of channels, 1=mono, 2=stereo
	long nSamplesPerSec;	//playback frequency
	long nAvgBytesPerSec;	//number of bytes per second to transfer
	short int nBlockAlign;
	short int FormatSpecific;	//format specific data area;
} WAVE;

typedef struct {
	char dID[4];		//should contain the word DATA
	long dLen;			//length of data in the dData field
} DATA;

struct WAVEFILE {
	RIFF riff;
	WAVE wave;
	DATA data;
};

class vco {
	long phaseaccum;
	long **inputs;
	int ninputs;
	int maxinputs;
	long output;
public:
	vco(int number_of_inputs);
	~vco();
	int Get(void);
	void Sync(void);
	void itterate(void);
	long *GetOutput();
	int SetInput(long *newinput);
};

#define MMFILTER_LOWPASS	0
#define MMFILTER_HIGHPASS	1
#define MMFILTER_BANDPASS	2

class mmfilter {
	long z1;		//delay accumulator #1
	long z2;		//delay accumulator #2
	long lp;		//low pass output
	long bp;		//band pass output
	long hp;		//high pass output
	long **freqinputs;	//array of frequency control inputs
	int nfreqin;			//number of frequency inputs
	int maxfreqin;			//maximum number of frequency inputs
	long **qinputs;		//array of q control inputs
	int nqin;				//number of q inputs
	int maxqin;				//maximum number of q inputs
	long **vinputs;		//array of signal inputs
	int nvin;				//number of signal inputs
	int maxvin;				//maximum number of singal inputs
public:
	mmfilter(int nf,int nq,int nv);
	~mmfilter();
	int SetVinput(long *v);
	int SetQinput(long *q);
	int SetFinput(long *f);
	long *GetOutput(int s);		//select an output
	void itterate(void);
};

class vca {
	long output;
	long **vinputs;
	int nvin;
	int maxvin;
	long **cinputs;
	int ncin;
	int maxcin;
public:
	vca(int nv,int nc);
	~vca();
	int SetVinput(long *v);
	int SetCinput(long *c);
	long *GetOutput(void);
	void itterate(void);
};

class adsr {
	long output;
	long z;	//time delay for low pass filter
	long **attack;		//controll for attack
	int nattack;		//number of attack inputs
	int maxattack;		//maximum number of attack inputs
	long **decay;		//control for decay
	int ndecay;			//number of decay inputs
	int maxdecay;		//maximum number of decay inputs
	long **sustain;	//control for sustain
	int nsustain;		//number of sustain inputs
	int maxsustain;	//maximum number of sustain inputs
	long **release;	//control for release
	int nrelease;		//number of release inputs
	int maxrelease;	//maximum number of release inputs
	int *gate;
	int *trig;
	int state;
public:
	adsr(int na,int nd,int ns,int nr);
	~adsr();
	int SetAttack(long *v);
	int SetDecay(long *v);
	int SetSustain(long *v);
	int SetRelease(long *v);
	void SetGate(int *g);
	void SetTrig(int *t);
	long *GetOutput(void);
	void itterate(void);
};

class Output {
	long **inputs;
	int ninputs;
	int maxinputs;
	FILE *outfile;
public:
	Output(int ni,FILE *out);
	~Output();
	int SetInput(long *in);
	void itterate(void);
};

class Dac {
	long output;
	int gate;
	int trig;
	int oldtrig;
public:
	Dac();
	~Dac();
	void SetFreq(double F);
	void SetTrig(void);
	void SetGate(int flag);
	long *GetOutput(void);
	int *GetGate(void);
	int *GetTrig(void);
	void itterate(void);
};

class pot {
	long output;
public:
	pot(){output=0;}
	~pot(){}
	long *GetOutput(void){return &output;}
	void SetPot(long v){output = v;}
};

